unit uMain;

interface

uses
  Windows, AIMPSDKAddons, AIMPSDKCore, AIMPAddonCustomPlugin, uPlaybackQueue;

type

  { TAIMPShuffleBasedOnRatingPlugin }

  TAIMPShuffleBasedOnRatingPlugin = class(TAIMPAddonsCustomPlugin,
    IAIMPAddonsPlaylistManagerListener,
    IAIMPAddonsPlayingQueueController,
    IAIMPCoreUnitMessageHook)
  private
    FActive: Boolean;
    FPlaybackQueue: TAIMPShufflePlaybackQueue;
    FPlayerManager: IAIMPAddonsPlayerManager;
    FPlaylistManager: IAIMPAddonsPlaylistManager;
    procedure SetActive(AValue: Boolean);
  protected
    // IAIMPAddonPlugin
    function Finalize: HRESULT; override; stdcall;
    function Initialize(ACoreUnit: IAIMPCoreUnit): HRESULT; override; stdcall;
    function GetPluginAuthor: PWideChar; override; stdcall;
    function GetPluginInfo: PWideChar; override; stdcall;
    function GetPluginName: PWideChar; override; stdcall;

    // IAIMPAddonsPlaylistManagerListener
    procedure StorageActivated(ID: HPLS); stdcall;
    procedure StorageAdded(ID: HPLS); stdcall;
    procedure StorageChanged(ID: HPLS; AFlags: Cardinal); stdcall;
    procedure StorageRemoved(ID: HPLS); stdcall;

    // IAIMPAddonsPlayingQueueController
    function GetFile(var ID: HPLS; var AEntryIndex: Integer; ANext: LongBool): HRESULT; stdcall;

    // IAIMPCoreUnitMessageHook
    procedure CoreMessage(AMessage: Cardinal; AParam1: Integer; AParam2: Pointer; var AResult: HRESULT); stdcall;
  public
    property Active: Boolean read FActive write SetActive;
    property PlaybackQueue: TAIMPShufflePlaybackQueue read FPlaybackQueue;
    property PlayerManager: IAIMPAddonsPlayerManager read FPlayerManager;
    property PlaylistManager: IAIMPAddonsPlaylistManager read FPlaylistManager;
  end;

implementation

uses
  SysUtils;

{ TAIMPShuffleBasedOnRatingPlugin }

function TAIMPShuffleBasedOnRatingPlugin.Finalize: HRESULT;
begin
  if CoreUnit <> nil then
    CoreUnit.MessageUnhook(Self);
  if FPlayerManager <> nil then
  begin
    FPlayerManager.PlayingQueueController(Self, False);
    FPlayerManager := nil;
  end;
  if FPlaylistManager <> nil then
  begin
    FPlaylistManager.ListenerRemove(Self);
    FPlaylistManager := nil;
  end;
  FreeAndNil(FPlaybackQueue);
  Result := inherited Finalize;
end;

function TAIMPShuffleBasedOnRatingPlugin.Initialize(ACoreUnit: IAIMPCoreUnit): HRESULT;
var
  APlayerManager: IAIMPAddonsPlayerManager;
  APlaylistManager: IAIMPAddonsPlaylistManager;
begin
  Result := inherited Initialize(ACoreUnit);
  if GetPlaylistManager(APlaylistManager) then
  begin
    FPlaylistManager := APlaylistManager;
    FPlaylistManager.ListenerAdd(Self);
    FPlaybackQueue := TAIMPShufflePlaybackQueue.Create(PlaylistManager);
    if GetPlayerManager(APlayerManager) then
    begin
      FPlayerManager := APlayerManager;
      FPlayerManager.PlayingQueueController(Self, True);
    end;
    if CoreUnit <> nil then
      CoreUnit.MessageHook(Self);
    Active := CorePropertyGetValueB(AIMP_MSG_PROPERTY_SHUFFLE, False);
  end;
end;

function TAIMPShuffleBasedOnRatingPlugin.GetPluginAuthor: PWideChar;
begin
  Result := 'Artem Izmaylov';
end;

function TAIMPShuffleBasedOnRatingPlugin.GetPluginInfo: PWideChar;
begin
  Result := 'Shuffle playback based on files rating';
end;

function TAIMPShuffleBasedOnRatingPlugin.GetPluginName: PWideChar;
begin
  Result := 'Shuffle Based On Rating';
end;

procedure TAIMPShuffleBasedOnRatingPlugin.StorageActivated(ID: HPLS);
begin
  // do nothing
end;

procedure TAIMPShuffleBasedOnRatingPlugin.StorageAdded(ID: HPLS);
begin
  // do nothing
end;

procedure TAIMPShuffleBasedOnRatingPlugin.StorageChanged(ID: HPLS; AFlags: Cardinal);
begin
  if Active and (ID = PlaybackQueue.Storage) then
  begin
    if AFlags and AIMP_PLAYLIST_NOTIFY_CONTENT <> 0 then
      PlaybackQueue.UpdateContent;
    if AFlags and AIMP_PLAYLIST_NOTIFY_PLAYINDEX <> 0 then
      PlaybackQueue.UpdateCursor;
  end;
end;

procedure TAIMPShuffleBasedOnRatingPlugin.StorageRemoved(ID: HPLS);
begin
  if ID = PlaybackQueue.Storage then
    PlaybackQueue.Storage := nil;
end;

function TAIMPShuffleBasedOnRatingPlugin.GetFile(var ID: HPLS; var AEntryIndex: Integer; ANext: LongBool): HRESULT;
begin
  Result := E_FAIL;
  if Active then
  begin
    PlaybackQueue.Storage := ID;
    if not PlaybackQueue.GetFile(AEntryIndex, ANext) then
    begin
      //todo: AIMP_MSG_PROPERTY_ACTION_ON_END_OF_PLAYLIST
    end;
    Result := S_OK;
  end;
end;

procedure TAIMPShuffleBasedOnRatingPlugin.CoreMessage(
  AMessage: Cardinal; AParam1: Integer; AParam2: Pointer; var AResult: HRESULT);
begin
  case AMessage of
    AIMP_MSG_EVENT_PROPERTY_VALUE:
      if AParam1 = AIMP_MSG_PROPERTY_SHUFFLE then
        Active := PLongBool(AParam2)^;
  end;
end;

procedure TAIMPShuffleBasedOnRatingPlugin.SetActive(AValue: Boolean);
begin
  if FActive <> AValue then
  begin
    FActive := AValue;
    if Active then
      PlaybackQueue.Storage := PlaylistManager.StoragePlayingGet
    else
      PlaybackQueue.Storage := nil;
  end;
end;

end.
